@@ -39,19 +39,16 @@ from consolekit.utils import abort
 # this package
 from dep_checker import check_imports
 
-__all__ = ["main"]
+__all__ = ("main", )
 
 
-@click.argument(
-		"pkg-name",
-		type=click.STRING,
-		)
+@colour_option()
 @click.option(
-		"--req-file",
+		"-d",
+		"--work-dir",
 		type=click.STRING,
-		metavar="FILENAME",
-		default="requirements.txt",
-		help="The requirements file.",
+		default='.',
+		help="The directory to find the source of the package in. Useful with the src/ layout."
 		)
 @click.option(
 		"-a",
@@ -61,13 +58,16 @@ __all__ = ["main"]
 		help="Requirements which are allowed to be unused in the source code."
 		)
 @click.option(
-		"-d",
-		"--work-dir",
+		"--req-file",
+		type=click.STRING,
+		metavar="FILENAME",
+		default="requirements.txt",
+		help="The requirements file.",
+		)
+@click.argument(
+		"pkg-name",
 		type=click.STRING,
-		default='.',
-		help="The directory to find the source of the package in. Useful with the src/ layout."
 		)
-@colour_option()
 @click_command()
 def main(
 		pkg_name: str,
@@ -75,7 +75,7 @@ def main(
 		allowed_unused: Optional[List[str]],
 		colour: Optional[bool],
 		work_dir: str = '.',
-		):
+		) -> None:
 	"""
 	Tool to check all requirements are actually required.
 	"""
@@ -84,13 +84,14 @@ def main(
 		allowed_unused = None
 
 	try:
-		sys.exit(check_imports(
+		ret = check_imports(
 				pkg_name,
 				req_file=req_file,
 				allowed_unused=allowed_unused,
 				colour=colour,
 				work_dir=work_dir,
-				))
+				)
+		sys.exit(ret)
 	except FileNotFoundError as e:
 		raise abort(str(e))
 
